﻿/**
 * Copyright (c) 2006, Opera Software ASA
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Opera Software ASA nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY OPERA SOFTWARE ASA AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL OPERA SOFTWARE ASA AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var feed;
window.onload = function() {
	feed = new Feed (skeletonConfig['feedURL'], skeletonConfig['feedTitle'], skeletonConfig['feedDescr']);
	window['controller'] = new Controller();
	feed.setUpdateInterval(15);
	feed.setDataHandler( parsers[ skeletonConfig['feedVersion'] ] );
	feed.addItemListener( onNewItemsHandler );
	feed.setMaxItems(skeletonConfig['maxItems']);
	feed.forceUpdate();
	
	document.title = skeletonConfig['feedTitle'];
	document.getElementById('widgetTitle').firstChild.nodeValue = skeletonConfig['feedTitle'];

	var baseHref = document.createElement('base');
	baseHref.href = skeletonConfig['feedURL'];
	document.getElementsByTagName('head')[0].appendChild(baseHref);
	
	window.resizeTo( 360, 260 );
}

function onNewItemsHandler(noChange, err) {
	var container = document.getElementById('itemList');
	var list = feed.getItemList();

	if (err) {
    	if (container.firstChild && container.firstChild.id == 'noContent') {
    		return;
    	}

		var li = document.createElement('li');
		li.appendChild( document.createTextNode( '(Unable to display feed)' ) );
		li.appendChild( document.createElement( 'br' ) );
		li.appendChild( document.createTextNode( err ) );
		container.insertBefore( li, container.firstChild );
		li.id = 'noContent';
		return;
	}

	if (list.length == 0) {
		container.innerHTML = '';
		var li = document.createElement('li');
		li.appendChild( document.createTextNode( '(No feed items found)' ) );
		container.appendChild(li);
		li.id = 'noContent';
		return;
	}
	
	// Unchanged feed, leave it alone
    if (noChange) return;
		
	// Display feed items
	container.innerHTML = '';
	var pointer = null, li = null, content = null;
	
	for (var i = 0; pointer = list[i]; i++) {
		li = document.createElement('li');
		li.className = pointer.read ? 'read' : '';
		li.guid = pointer.getGUID();
		container.appendChild( li );
		
		var tit = pointer.getTitle() || '(No title)';
		if ( typeof tit == 'string' ) tit = document.createTextNode( tit );
		var h2 = document.createElement('h2');
		h2.className = 'postTitle';
		h2.appendChild( tit );
		li.appendChild( h2 );
		
		var content = document.createElement('div');
		var h3 = document.createElement('h3');
		h3.className = 'dateTime';
		/* Normal - no toAge */
		var pub = pointer.getDate() || '(No date)';
		if (typeof pub != 'string') {
			pub = pub.getDate() + ' ' + months[pub.getMonth()] + ', ' +
				padZero(pub.getHours()) + ':' + padZero(pub.getMinutes());
		}
		h3.appendChild( document.createTextNode( pub ) );
		/* End Normal */
		content.appendChild( h3 );
		li.appendChild( content );
		li.expander = content;
		
		var desc = pointer.getDesc() || document.createTextNode( '(No text)' );
		if ( typeof desc == 'string' ) desc = document.createTextNode( desc );
		content.appendChild( desc );
		
		var link, href = pointer.getLink();
		if ( href ) {
			link = document.createElement('a');
			link.href = href;
			link.appendChild( document.createTextNode( 'Go to entry' ) );
		} else {
			link = document.createTextNode( '(No link)' );
		}
		link.className = 'morelink';
		content.appendChild(link);
		
	}

    updateUnread();	

}
	
function Controller() {
	function clickHandler(event) {
		var el = event.target;
		event.stopPropagation();
		
		while (el) {
			switch (el.nodeName.toLowerCase()) {
				case 'h2' :
					if (el.className == 'postTitle') {
						clickOnTitleHandler(el.parentNode);
						return;
					}
					break;
				case 'a' :
					// Happens automatically
					//widget.openURL(el.getAttribute('href'));
					return;
				case 'ul' :
					if (el.id == 'itemList') {
						return;
					}
					break;
				default :
					break;
			}
			el = el.parentNode;
		}
	}
	
	function clickOnTitleHandler(ele) {
		var adiv = ele.expander;
		var anim = adiv.aniobj;
		
		if (! anim) {
			anim = adiv.aniobj = adiv.createAnimation();
			anim.adiff = Math.min( 250, adiv.scrollHeight );
			anim.bdiff = 0;
			anim.speed = 20;
			anim.accelerationProfile = Animation.prototype.sine;
			anim.style.height = '0px';
			anim.onfinish = function() {
				adiv.style.overflow = '';
				if ( ! ele.hasClass('active') ) {
					window.resizeBy( 0, - Math.max( 0, anim.adiff - anim.bdiff ) );
					anim.bdiff = 0;
				}
			}
			anim.onstart = function() {
				adiv.style.overflow = 'hidden';
				if ( ele.hasClass('active') ) {
					window.resizeBy( 0, Math.max( 0, anim.adiff - anim.bdiff ) );
					anim.bdiff = 0;
				}
			}
		}
		
		if ( ele.hasClass('active') ) {
			ele.dropClass('active');
			anim.addAnimation('height', anim.style.height, '0px').run();
		} else {
			var all = ele.parentNode.getElementsByTagName('li');
			var pointer;
			
			for (var i = 0; pointer = all[i]; i++) {
				if ( pointer.hasClass('active') ) {
					pointer.dropClass('active');
					var bnim = pointer.expander.aniobj;
					bnim.bdiff = anim.adiff;
					anim.bdiff = bnim.adiff;
					bnim.addAnimation('height', bnim.style.height, '0px').run();
					break;
				}
			}
			
			feed.setItemRead(ele.guid);
			ele.addClass('active');
			ele.addClass('read');
			updateUnread();
			
			anim.addAnimation('height', '0px', anim.adiff + 'px').run();
		}
	}
	
	var minimized = false;
	this.expandMinimize = function() {
		var ele = document.getElementById('content');
		var animation = ele.animation;
		
        if (! animation) {
			animation = ele.animation = ele.createAnimation();
			animation.accelerationProfile = Animation.prototype.sine;
			animation.speed = 10;
			animation.style.height = '0px';
			animation.onfinish = function() {
				if ( minimized ) {
                    window.resizeBy( 0, -ele.scrollHeight );
                } else {
                	ele.style.height = 'auto';
                }
			}
			animation.onstart = function() {
				if (! minimized) {
                    window.resizeBy( 0, ele.scrollHeight );
                }
			}
		}

		if ( minimized ) {
            minimized = false; 
			animation.addAnimation('height', '0px', ele.scrollHeight + 'px').run();
		} else {
			minimized = true;
			animation.addAnimation('height', ele.scrollHeight + 'px', '0px').run();
		}
	}

	document.getElementById('itemList').addEventListener('click', clickHandler, true);
	document.getElementById('expandMinimize').addEventListener( 'click', this.expandMinimize, false );
	document.getElementById('flip').addEventListener( 'click', flipSide, false );
	document.getElementById('disanim').addEventListener( 'click', function() { Animation.on = ! this.checked; }, false );
}

function updateUnread(num) {
	document.getElementById('unread').firstChild.nodeValue = feed.getUnreadCount();
}

var flipped = false;
function flipSide() {

    var header = document.getElementById('widgetTitle');
    var itemList = document.getElementById('itemList');
    var back  = document.getElementById('back');
    
    var con = document.getElementById('content');
    var before = con.offsetHeight;
    
    if (flipped) {
        itemList.style.display = '';
        back.style.display = '';
        header.firstChild.nodeValue = skeletonConfig['feedTitle'];
        flipped = false;
    } else {
        itemList.style.display = 'none';
        back.style.display = 'block';
        header.firstChild.nodeValue = 'About Widgetize!';
        flipped = true;
    }
    
    var after = con.offsetHeight;
    window.resizeBy( 0, after - before );
}
